// Registers the preferences with RTB, if you have it.
if(isFile("Add-Ons/System_ReturnToBlockland/server.cs"))
{
	// Makes sure the thing is running.
	if(!$RTB::RTBR_ServerControl_Hook)
	{
		exec("Add-Ons/System_ReturnToBlockland/RTBR_ServerControl_Hook.cs");
	}
	
	// Description, ModName, Variable, Properties, Mod, Default value, Requires Restart, Super-Admin Only, Callback Function
	RTB_registerPref("Enable Health Bar", "Health Settings", "$HealthBar::Enabled", "bool", "Script_Healthbar", true, false, false, "Healthbar_UpdateMethod");
	RTB_registerPref("Minigames Only?", "Health Settings", "$HealthBar::MiniOnly", "bool", "Script_Healthbar", true, false, false, "Healthbar_UpdateMinigame");
}
// If you not have RTB, the mod registers it to the most likely preferred settings.
else
{
	$HealthBar::Enabled = true;
	$HealthBar::MiniOnly = true;
}

// Notifies server and removes all healthbars.
function Healthbar_UpdateMethod(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', "\c3+ Healthbar\c0 - The Healthbar mod is now " @ (%newVal ? "\c2Enabled" : "\c3Disabled") @ "\c0.");
	
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
	{
		%client = ClientGroup.getObject(%a);
		
		if(%client.hasHealthBar)
		{
			%player = %client.player;
			
			if(isObject(%player))
			{
				%canSee = ($HealthBar::Enabled && (!%newVal || isObject(%client.minigame)) ? true : false);
				
				commandToClient(%client, 'setHealthBarVisible', %canSee);
				
				if(%canSee)
				{
					%dmgPercent = %player.getDamageLevel() - (%player.pseudoHealth + %player.tf2Overheal);
					%dmgPercent = %dmgPercent / %player.getDatablock().maxDamage;
				}
				else
				{
					%dmgPercent = 0;
				}
				
				commandToClient(%client, 'setDamageLevel', %dmgPercent);
			}
		}
	}
}

// Notifies server and removes all healthbars.
function Healthbar_UpdateMinigame(%oldVal, %newVal)
{
	schedule(0, 0, "messageAll", '', "\c3+ Healthbar\c0 - The Healthbar mod is now on " @ (!%newVal ? "\c3All the Time" : "\c3in Minigames") @ "\c0.");
	
	for(%a = 0; %a < ClientGroup.getCount(); %a++)
	{
		%client = ClientGroup.getObject(%a);
		
		if(%client.hasHealthBar)
		{
			%player = %client.player;
			
			if(isObject(%player))
			{
				%canSee = ($HealthBar::Enabled && (!%newVal || isObject(%client.minigame)) ? true : false);
				
				commandToClient(%client, 'setHealthBarVisible', %canSee);
				
				if(%canSee)
				{
					%dmgPercent = %player.getDamageLevel() - (%player.pseudoHealth + %player.tf2Overheal);
					%dmgPercent = %dmgPercent / %player.getDatablock().maxDamage;
				}
				else
				{
					%dmgPercent = 0;
				}
				
				commandToClient(%client, 'setDamageLevel', %dmgPercent);
			}
		}
	}
}

// This gives the client a tracer when it registers the mod with the server.
// It also forces the healthbar to appear if the conditions are met.
function serverCmdRegisterHealthBarMod(%client)
{
	%client.hasHealthBar = true;
	
	// If the mod is on...
	if($HealthBar::Enabled)
	{
		// And the Client has a Player..
		%player = %client.player;
		
		if(isObject(%player))
		{
			// And Minigames Only + Client is in Minigame OR Not just minigames.
			if(!$HealthBar::MiniOnly || isObject(%client.minigame))
			{
				// Show the bar.
				%dmgPercent = %player.getDamageLevel() - (%player.pseudoHealth + %player.tf2Overheal);
				%dmgPercent = %dmgPercent / %player.getDatablock().maxDamage;
				
				commandToClient(%client, 'setHealthBarVisible', true);
				commandToClient(%client, 'setDamageLevel', %dmgPercent);
			}
			else
			{
				// Hide the bar.
				commandToClient(%client, 'setHealthBarVisible', false);
				commandToClient(%client, 'setDamageLevel', 0);
			}
		}
	}
}

package Healthbar_Server
{
	// onDamage also handles negative integers.
	function Armor::onDamage(%this, %obj, %damage)
	{
		parent::onDamage(%this, %obj, %damage);
		
		// If the mod is on...
		if($HealthBar::Enabled)
		{
			// And the client exists and has the healthbar mod...
			%client = %obj.client;
			
			if(isObject(%client) && %client.hasHealthBar)
			{
				// And Minigames Only + Client is in Minigame OR Not just minigames.
				if(!$HealthBar::MiniOnly || isObject(%client.minigame))
				{
					// Send their health.
					%dmgPercent = %obj.getDamageLevel() - (%obj.pseudoHealth + %obj.tf2Overheal);
					%dmgPercent = mFloatLength(%dmgPercent / %obj.getDatablock().maxDamage, 4);
					
					commandToClient(%client, 'setDamageLevel', %dmgPercent);
				}
			}
		}
	}
	
	function GameConnection::onClientEnterGame(%client)
	{
		parent::onClientEnterGame(%client);
		commandToClient(%client, 'detectHealthBarMod');
	}
	
	function GameConnection::spawnPlayer(%client)
	{
		%result = parent::spawnPlayer(%client);
		
		if(%client.hasHealthBar && $HealthBar::Enabled && (!$HealthBar::MiniOnly || isObject(%client.minigame)))
		{
			%player = %client.player;
			
			if(isObject(%player))
			{
				%dmgPercent = %player.getDamageLevel() - (%player.pseudoHealth + %player.tf2Overheal);
				%dmgPercent = mFloatLength(%dmgPercent / %player.getDatablock().maxDamage, 4);
				
				commandToClient(%client, 'setHealthBarVisible', true);
				commandToClient(%client, 'setDamageLevel', %dmgPercent);
			}
		}
		
		return %result;
	}
	
	function Player::doOverhealDecay(%obj)
	{
		parent::doOverhealDecay(%obj);
		
		// If the mod is on...
		if($HealthBar::Enabled)
		{
			// And the client exists and has the healthbar mod...
			%client = %obj.client;
			
			if(isObject(%client) && %client.hasHealthBar)
			{
				// And Minigames Only + Client is in Minigame OR Not just minigames.
				if(!$HealthBar::MiniOnly || isObject(%client.minigame))
				{
					// Send their health.
					%dmgPercent = %obj.getDamageLevel() - (%obj.pseudoHealth + %obj.tf2Overheal);
					%dmgPercent = mFloatLength(%dmgPercent / %obj.getDatablock().maxDamage, 4);
					
					commandToClient(%client, 'setDamageLevel', %dmgPercent);
				}
			}
		}
	}
};
activatePackage(Healthbar_Server);
